/*
 * Licensed Materials - Property of IBM
 *
 * com.ibm.etools.c
 *
 *  Copyright IBM Corporation 2004. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.etools.c.importer;

import java.util.HashMap;

import com.ibm.etools.c.CDerivableType;
import com.ibm.etools.c.CFunction;
import com.ibm.etools.c.CStructured;

/**
 * The CTypeTable a place to hold all C model type objects that have been
 * created. When we need to find a type from a string representation, we first
 * search the type table to see whether there is already a C model object for
 * it. If it is not there, a new CClassifier object will be created and added to
 * the type table.
 * 
 * The CTypeTable uses four HashMaps to store the C datatypes, functions,
 * struct/union details and source locations.
 */

public class CTypeTable {

    public static final String copyright = "Licensed Material - Property of IBM\n com.ibm.etools.c\n(C) Copyright IBM Corp. 2004 - All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; //$NON-NLS-1$

    private CModelConverter _modelConverter;

    // A map to store the types
    private HashMap _typeTable = null;

    // A map to store the functions
    private HashMap _functionTable = null;

    // A map to store struct/union details
    private HashMap _structuredTable = null;

    // A map to store source locations
    private HashMap _sourceLocationTable = null;

    /**
     * Default constructor
     */
    public CTypeTable() {
        _modelConverter = null;
    }

    /**
     * Construct a CTypeTable object from a given CModelConverter.
     */
    public CTypeTable(CModelConverter modelConverter) {
        _modelConverter = modelConverter;
        _typeTable = new HashMap(8);
        _functionTable = new HashMap(4);
        _structuredTable = new HashMap(4);
        _sourceLocationTable = new HashMap();
    }

    /**
     * Return a CFunction object for a given function name.
     */
    public CFunction getFunction(String functionName) {
        return (CFunction) _functionTable.get(functionName);
    }

    /**
     * Add a function object to the type table.
     */
    public void addFunction(String functionName, CFunction function) {
        if (getFunction(functionName) == null) {
            _functionTable.put(functionName, function);
        }
    }

    /**
     * Return a CStructuredDetails object for a given struct/union.
     */
    public CStructuredDetails getStructuredDetails(CStructured astructured) {
        return (CStructuredDetails) _structuredTable.get(astructured);
    }

    /**
     * Add a CStructuredDetails object to the type table.
     */
    public void addStructuredDetails(CStructured astructured, CStructuredDetails details) {
        if (getStructuredDetails(astructured) == null) {
            _structuredTable.put(astructured, details);
        }
    }

    /**
     * Return the source location for a given object.
     */
    public CSourceLocation getSourceLocation(Object element) {
        return (CSourceLocation) _sourceLocationTable.get(element);
    }

    /**
     * Add an object and its source location to the type table.
     */
    public void addSourceLocation(Object element, CSourceLocation sourceLocation) {
        if (getSourceLocation(element) == null) {
            _sourceLocationTable.put(element, sourceLocation);
        }
    }

    /**
     * Add a type to the type table.
     */
    public void addType(String typeName, CDerivableType type) {

        // Add the type to the table only if it is not there already.
        //
        if (!_typeTable.containsKey(typeName)) {
            _typeTable.put(typeName, type);
        }
    }

    /**
     * Does the table contain this type?
     */
    public boolean containsType(CDerivableType type) {
        return _typeTable.containsValue(type);
    }

    /**
     * Return the type in this type table from a given name.
     */
    public CDerivableType getType(String typeName) {
        return (CDerivableType) _typeTable.get(typeName);
    }

}